searchData={"items":[{"type":"module","title":"Numscriptex","doc":"NumscriptEx is a library that allows its users to check and run [Numscripts](https://docs.formance.com/numscript/)\nvia Elixir.\n\nWant to check if your script is valid and ready to go? Use the `check/1` function.\nAlready checked the script and want to execute him? Use the `run/2` function.","ref":"Numscriptex.html"},{"type":"type","title":"Numscriptex.balances/0","doc":"","ref":"Numscriptex.html#t:balances/0"},{"type":"function","title":"Numscriptex.check/1","doc":"To use `check/1` you just need to pass your numscript as its argument.\nEx:\n\n```elixir\niex> script = \"send [USD/2 100] (source = @foo destination = @bar)\"\niex> Numscriptex.check(script)\n{:ok, %{script: script}}\n```\n\nIt could also return some warnings, infos or hints inside the map","ref":"Numscriptex.html#check/1"},{"type":"type","title":"Numscriptex.check_log/0","doc":"","ref":"Numscriptex.html#t:check_log/0"},{"type":"type","title":"Numscriptex.check_result/0","doc":"","ref":"Numscriptex.html#t:check_result/0"},{"type":"type","title":"Numscriptex.errors/0","doc":"","ref":"Numscriptex.html#t:errors/0"},{"type":"type","title":"Numscriptex.postings/0","doc":"","ref":"Numscriptex.html#t:postings/0"},{"type":"function","title":"Numscriptex.run/2","doc":"To use `run/2` your first argument must be your script, and the second must\nbe a `%Numscriptex.Run{}` (go to `Numscriptex.Run` module to see more) struct.\nEx:\n\n```elixir\niex> script = \"send [USD/2 100] (source = @foo destination = @bar)\"\n...> balances = %{\"foo\" => %{\"USD/2\" => 500, \"EUR/2\" => 300}}\n...>\n...> struct =\n...> Numscriptex.Run.new()\n...> |> Numscriptex.Run.put!(:balances, balances)\n...> |> Numscriptex.Run.put!(:metadata, %{})\n...> |> Numscriptex.Run.put!(:variables, %{})\n...>\n...> Numscriptex.run(script, struct)\n```","ref":"Numscriptex.html#run/2"},{"type":"type","title":"Numscriptex.run_result/0","doc":"","ref":"Numscriptex.html#t:run_result/0"},{"type":"function","title":"Numscriptex.version/0","doc":"`version/0` simply shows a map with both [Numscript-WASM](https://github.com/PagoPlus/numscript-wasm) and NumscriptEx versions. Ex:\n\n```elixir\niex> Numscriptex.version()\n%{numscriptex: \"v0.2.1\", numscript_wasm: \"v0.0.2\"}\n```","ref":"Numscriptex.html#version/0"},{"type":"module","title":"Numscriptex.AssetsManager","doc":"`Numscriptex.AssetsManager` is responsible for ensuring that the Numscriptex\nlibrary have all it needs (i.e. a WASM binary file that is used to check and run Numscripts)\nto run correctly at compile time.","ref":"Numscriptex.AssetsManager.html"},{"type":"function","title":"Numscriptex.AssetsManager.binary_path/0","doc":"","ref":"Numscriptex.AssetsManager.html#binary_path/0"},{"type":"macro","title":"Numscriptex.AssetsManager.ensure_wasm_binary_is_valid/0","doc":"","ref":"Numscriptex.AssetsManager.html#ensure_wasm_binary_is_valid/0"},{"type":"function","title":"Numscriptex.AssetsManager.hash_wasm_binary/0","doc":"","ref":"Numscriptex.AssetsManager.html#hash_wasm_binary/0"},{"type":"module","title":"Numscriptex.Balances","doc":"`Numscriptex.Balances` is responsible for building the account's final balance\nafter running your [Numscript](https://docs.formance.com/numscript/), so you\ncan see the results of all transactions.","ref":"Numscriptex.Balances.html"},{"type":"function","title":"Numscriptex.Balances.put/2","doc":"Receives the account assets (balance field from `%Numscriptex.Run{}`), and the\npostings that are generated after running the numscript transaction.\n\nThe result will be a map contaning the initial and final balances of each\naccount assets. Ex:\n\n```elixir\niex> account_assets = %{\n...>     \"foo\" => %{\n...>     \"USD/2\" => 500,\n...>     \"EUR/2\" => 300\n...>   }\n...> }\n...>\n...> postings = [\n...>   %{\n...>     \"amount\" => 100,\n...>     \"asset\" => \"USD/2\",\n...>     \"destination\" => \"bar\",\n...>     \"source\" => \"foo\"\n...>   }\n...> ]\n...>\n...> Numscriptex.Balances.put(account_assets, postings)\n[\n  %{\n    \"account\" => \"foo\",\n    \"asset\" => \"EUR/2\",\n    \"final_balance\" => 300,\n    \"initial_balance\" => 300\n  },\n  %{\n    \"account\" => \"foo\",\n    \"asset\" => \"USD/2\",\n    \"final_balance\" => 400,\n    \"initial_balance\" => 500\n  },\n  %{\n    \"account\" => \"bar\",\n    \"asset\" => \"USD/2\",\n    \"final_balance\" => 100,\n    \"initial_balance\" => 0\n  }\n]\n```","ref":"Numscriptex.Balances.html#put/2"},{"type":"module","title":"Numscriptex.CheckLog","doc":"After you check your numscript you might get a variety of logs even if it is valid,\n`Numscriptex.CheckLog` is responsible for standardize these logs.","ref":"Numscriptex.CheckLog.html"},{"type":"function","title":"Numscriptex.CheckLog.from_map/1","doc":"Get a map with log data about a checked numscript.\n\n```elixir\niex> map = %{\n...>   character: 10,\n...>   level:  :warning,\n...>   line: 1,\n...>   message: \"warning message\"\n...> }\n...>\n...> Numscriptex.CheckLog.from_map(map)\n%Numscriptex.CheckLog{\n  character: 10,\n  level:  :warning,\n  line: 1,\n  message: \"warning message\"\n}\n```","ref":"Numscriptex.CheckLog.html#from_map/1"},{"type":"type","title":"Numscriptex.CheckLog.log_levels/0","doc":"","ref":"Numscriptex.CheckLog.html#t:log_levels/0"},{"type":"type","title":"Numscriptex.CheckLog.t/0","doc":"Type that represents `Numscriptex.CheckLog` struct.","ref":"Numscriptex.CheckLog.html#t:t/0"},{"type":"type","title":"Fields - Numscriptex.CheckLog.t/0","doc":"* `:character` the character position where the log occurred\n* `:level` the log level\n* `:line` the line where the log occur\n* `:message` the log message","ref":"Numscriptex.CheckLog.html#t:t/0-fields"},{"type":"module","title":"Numscriptex.Run","doc":"A Numscript needs some other data aside from the script itself to run correctly,\nand `Numscriptex.Run` solves this problem. If you want to know what exactly these\nadditional fields are, you can learn more on [Numscript Playground](https://playground.numscript.org/?template=simple-send) and the [Numscript Docs](https://docs.formance.com/numscript/).","ref":"Numscriptex.Run.html"},{"type":"function","title":"Numscriptex.Run.new/0","doc":"Creates a new `Numscriptex.Run{}` struct.\n\n```elixir\niex>  Numscriptex.Run.new()\n%Numscriptex.Run{\n  variables: %{},\n  balances: %{},\n  metadata: %{}\n}\n```","ref":"Numscriptex.Run.html#new/0"},{"type":"function","title":"Numscriptex.Run.put/3","doc":"Puts the chosen value under the field key on the `Numscriptex.Run` struct as\nlong both are valid.\n\n```elixir\niex> struct = Numscriptex.Run.new()\n...> balances =  %{\"foo\" => %{\"USD/2\" => 500, \"EUR/2\" => 300}}\n...>\n...> Numscriptex.Run.put(struct, :balances, balances)\n{:ok,\n  %Numscriptex.Run{\n    balances: %{\"foo\" => %{\"USD/2\" => 500, \"EUR/2\" => 300}},\n    variables: %{},\n    metadata: %{}\n  }\n}\n```\nIf the value or the field key are invalid, `put/3` will return a 3 element\nerror tuple. Ex:\n\n```elixir\niex> struct = Numscriptex.Run.new()\n...>\n...> Numscriptex.Run.put(struct, :invalid_field, %{})\n{:error, :invalid_field, %{details: \"The field 'invalid_field' does not exists.\"}}\n```","ref":"Numscriptex.Run.html#put/3"},{"type":"function","title":"Numscriptex.Run.put!/3","doc":"Same as `put/3`, but raises an exception if any argument is invalid.\n\n```elixir\niex> struct = Numscriptex.Run.new()\n...> balances =  [%{\"foo\" => %{\"USD/2\" => 500, \"EUR/2\" => 300}}]\n...>\n...> Numscriptex.Run.put!(struct, :balances, balances)\n** (ArgumentError) Argument `value` must be a map.\n```","ref":"Numscriptex.Run.html#put!/3"},{"type":"type","title":"Numscriptex.Run.t/0","doc":"Type that represents `Numscriptex.Run` struct.","ref":"Numscriptex.Run.html#t:t/0"},{"type":"type","title":"Fields - Numscriptex.Run.t/0","doc":"* `:balances` a map with account's assets balances\n* `:metadata` [metada variables](https://docs.formance.com/numscript/reference/metadata)\n* `:variables` [variables](https://docs.formance.com/numscript/reference/variables) used inside the script","ref":"Numscriptex.Run.html#t:t/0-fields"},{"type":"module","title":"Numscriptex.Utilities","doc":"`Numscriptex.Utilities` module contain reusable code that are common in more than one module\nof this library.","ref":"Numscriptex.Utilities.html"},{"type":"function","title":"Numscriptex.Utilities.normalize_keys/2","doc":"Convert map keys to the desired type. Can go from string to atom and vice versa.\n\n```elixir\niex> map = %{\"foo\" => 100}\n...> Utilities.normalize_keys(map, :atom)\n%{foo: 100}\n\niex> map = %{foo: 100}\n...> Utilities.normalize_keys(map, :string)\n%{\"foo\" => 100}\n```","ref":"Numscriptex.Utilities.html#normalize_keys/2"},{"type":"extras","title":"NumscriptEx","doc":"# NumscriptEx\nNumscriptEx is a library that allows its users to check and run Numscripts in Elixir. If this is your first time hearing about\nNumscripts, here is a quick explanation:\n\n[Numscript](https://docs.formance.com/numscript/) is a DSL made by [Formance](https://www.formance.com/)\nthat simplifies complex financial transactions with scripts that are easy to read,\nso you don't need a big, complex and error-prone codebase to deal with your finances.\n\nYou can see and execute some examples at the [Numscript Playground](https://playground.numscript.org/?template=simple-send).","ref":"readme.html"},{"type":"extras","title":"Installation - NumscriptEx","doc":"You will just need to add `:numscriptex` as a dependency on your `mix.exs`, and run the `mix deps.get` command:\n\n```elixir\ndef deps do\n  [\n    {:numscriptex, \"~> 0.2.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - NumscriptEx","doc":"NumscriptEx needs some external assets ([Numscript-WASM](https://github.com/PagoPlus/numscript-wasm)),\nand you can override the default version.\n\nAvailable configurations:\n- `:version` the binary release version to use (see [numscript-wasm releases](https://github.com/PagoPlus/numscript-wasm/releases/)).\n- `:retries` number of times to retry the download in case of a network failure.\n\nEx:\n```elixir\nconfig :numscriptex,\n       version: \"0.0.2\",\n       retries: 3\n```\nThese above are the default values.","ref":"readme.html#configuration"},{"type":"extras","title":"Usage - NumscriptEx","doc":"This library basically has two core functions: `Numscriptex.check/1` and `Numscriptex.run/2`.\n\nWant to check if your script is valid and ready to go? Use the `check/1` function.\nAlready checked the script and want to execute it? Use the `run/2` function.\n\nBut before introducing these two functions, you will need to know what is the `Numscriptex.Run` struct.","ref":"readme.html#usage"},{"type":"extras","title":"Numscriptex.Run - NumscriptEx","doc":"A numscript needs some other data aside the script itself to run correctly, and\n`Numscriptex.Run` solves this problem.\n\nIf you want to know what exactly these additional data are, you can see the\n[Numscript Playground](https://playground.numscript.org/?template=simple-send) for examples.\n\nThe abstraction is made by creating a struct:\n```elixir\niex>  %Numscriptex.Run{\n...>    balances: %{},\n...>    metadata: %{},\n...>    variables: %{}\n...>  }\n```\nWhere:\n- `:balances` a map with the account's assets balances.\n- `:metadata` [metada variables](https://docs.formance.com/numscript/reference/metadata);\n- `:variables` [variables](https://docs.formance.com/numscript/reference/variables) used inside the script.\n\nAnd to create a new struct, you can use the `Numscriptex.Run.put/3` or `Numscriptex.Run.put!/3` functions. Ex:\n```elixir\niex>  variables = %{\"order\" => \"orders:2345\"}\n...>  balances = %{\"orders:2345\" => %{\"USD/2\" => 1000}}\n...>  metadata = %{\n...>    \"merchants:1234\" => %{\"commission\" => \"15%\"},\n...>    \"orders:2345\" => %{\"merchant\" => \"merchants:1234\"}\n...>  }\n...>\n...>  Numscriptex.Run.new()\n...>  |> Numscriptex.Run.put!(:balances, balances)\n...>  |> Numscriptex.Run.put!(:metadata, metadata)\n...>  |> Numscriptex.Run.put!(:variables, variables)\n```\n\nWill return:\n\n```elixir\niex> %Numscriptex.Run{\n...>   variables: %{\"orders:2345\" => %{\"USD/2\" => 1000}},\n...>   balances: %{\"order\" => \"orders:2345\"},\n...>   metadata: %{\n...>     \"merchants:1234\" => %{\"commission\" => \"15%\"},\n...>     \"orders:2345\" => %{\"merchant\" => \"merchants:1234\"}\n...>   }\n...> }\n```\n\nKindly reminder: you will always need a valid `Numscriptex.Run` struct to successfully execute your scripts.","ref":"readme.html#numscriptex-run"},{"type":"extras","title":"Check - NumscriptEx","doc":"To use `Numscriptex.check/1` you just have to pass your numscript as it's argument. Ex:\n\n```elixir\niex>  \"tmp/script.num\"\n...>  |> File.read!()\n...>  |> Numscriptex.check()\n{:ok, %{script: script}\n```\n\nYou don´t need to necessarily read from a file, as long as it is a string it's fine.\n\nSometimes, even if your script is valid, it could also return some warnings, infos or hints inside the map.\nEx:\n```elixir\niex> {:ok, %{\n...>     script: \"your numscript here\",\n...>     warnings: [\n...>             %CheckLog{\n...>               character: 10,\n...>               level: :warning,\n...>               line: 1,\n...>               message: \"warning message\"\n...>             }\n...>           ],\n...>     hints: [\n...>             %CheckLog{\n...>               character: 2,\n...>               level: :hint,\n...>               line: 7,\n...>               message: \"hint message\"\n...>             }\n...>           ],\n...>     infos: [\n...>             %CheckLog{\n...>               character: 9,\n...>               level: :info,\n...>               line: 14,\n...>               message: \"info message\"\n...>             }\n...>           ]\n...>   }\n...> }\n```\nThe `:script` is the only field that will always return if your script is valid, the other three are optional.","ref":"readme.html#check"},{"type":"extras","title":"Run - NumscriptEx","doc":"To use `Numscriptex.run/2` your first argument must be your script (the same you used in `Numscriptex.check/1`), and the second must be the `%Numscriptex.Run{}` struct. Ex:\n\n```elixir\niex>  Numscriptex.run(script, struct)\n{:ok, result}\n```\n\nWhere result will be something like this:\n```elixir\niex> %{\n...>   postings: [\n...>           %{\n...>             amount: 100,\n...>             asset: \"USD/2\",\n...>             destination: \"bar\",\n...>             source: \"foo\"\n...>           }\n...>         ],\n...>   balances: [\n...>           %{\n...>             account: \"foo\",\n...>             asset: \"EUR/2\",\n...>             final_balance: 300,\n...>             initial_balance: 300\n...>           },\n...>           %{\n...>             account: \"foo\",\n...>             asset: \"USD/2\",\n...>             final_balance: 400,\n...>             initial_balance: 500\n...>           },\n...>           %{\n...>             account: \"bar\",\n...>             asset: \"USD/2\",\n...>             final_balance: 100,\n...>             initial_balance: 0\n...>           }\n...>         ],\n...>   accountMeta: %{}\n...>   txMeta: %{}\n...> }\n```","ref":"readme.html#run"},{"type":"extras","title":"License - NumscriptEx","doc":"Copyright (c) 2025 MedFlow\n\nThis library is MIT licensed. See the [LICENSE](https://github.com/PagoPlus/numscriptex/blob/main/README.md) for details.","ref":"readme.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}